from .base_classes import *
from .support_functions import *


class CarePlan(Resource):
    """Describes the intention of how one or more practitioners intend to
    deliver care for a particular patient for a period of time,
    possibly limited to care for a specific condition or set of
    conditions.If the element is present, it must have either a
    @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, patient=None, status=None, period=None, modified=None, concern=None, participant=None, goal=None, activity=None, notes=None):
        self.original_tagname_ = None
        super(CarePlan, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.patient = patient
        self.status = status
        self.period = period
        self.modified = modified
        if concern is None:
            self.concern = []
        else:
            self.concern = concern
        if participant is None:
            self.participant = []
        else:
            self.participant = participant
        if goal is None:
            self.goal = []
        else:
            self.goal = goal
        if activity is None:
            self.activity = []
        else:
            self.activity = activity
        self.notes = notes
    def factory(*args_, **kwargs_):
        if CarePlan.subclass:
            return CarePlan.subclass(*args_, **kwargs_)
        else:
            return CarePlan(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_patient(self): return self.patient
    def set_patient(self, patient): self.patient = patient
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_period(self): return self.period
    def set_period(self, period): self.period = period
    def get_modified(self): return self.modified
    def set_modified(self, modified): self.modified = modified
    def get_concern(self): return self.concern
    def set_concern(self, concern): self.concern = concern
    def add_concern(self, value): self.concern.append(value)
    def insert_concern(self, index, value): self.concern[index] = value
    def get_participant(self): return self.participant
    def set_participant(self, participant): self.participant = participant
    def add_participant(self, value): self.participant.append(value)
    def insert_participant(self, index, value): self.participant[index] = value
    def get_goal(self): return self.goal
    def set_goal(self, goal): self.goal = goal
    def add_goal(self, value): self.goal.append(value)
    def insert_goal(self, index, value): self.goal[index] = value
    def get_activity(self): return self.activity
    def set_activity(self, activity): self.activity = activity
    def add_activity(self, value): self.activity.append(value)
    def insert_activity(self, index, value): self.activity[index] = value
    def get_notes(self): return self.notes
    def set_notes(self, notes): self.notes = notes
    def hasContent_(self):
        if (
            self.identifier or
            self.patient is not None or
            self.status is not None or
            self.period is not None or
            self.modified is not None or
            self.concern or
            self.participant or
            self.goal or
            self.activity or
            self.notes is not None or
            super(CarePlan, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlan', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlan', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlan'):
        super(CarePlan, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan')
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlan', fromsubclass_=False, pretty_print=True):
        super(CarePlan, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.patient is not None:
            self.patient.export(outfile, level, namespace_, name_='patient', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.period is not None:
            self.period.export(outfile, level, namespace_, name_='period', pretty_print=pretty_print)
        if self.modified is not None:
            self.modified.export(outfile, level, namespace_, name_='modified', pretty_print=pretty_print)
        for concern_ in self.concern:
            concern_.export(outfile, level, namespace_, name_='concern', pretty_print=pretty_print)
        for participant_ in self.participant:
            participant_.export(outfile, level, namespace_, name_='participant', pretty_print=pretty_print)
        for goal_ in self.goal:
            goal_.export(outfile, level, namespace_, name_='goal', pretty_print=pretty_print)
        for activity_ in self.activity:
            activity_.export(outfile, level, namespace_, name_='activity', pretty_print=pretty_print)
        if self.notes is not None:
            self.notes.export(outfile, level, namespace_, name_='notes', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlan', mapping_=None):
        element = super(CarePlan, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.patient is not None:
            patient_ = self.patient
            patient_.to_etree(element, name_='patient', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.period is not None:
            period_ = self.period
            period_.to_etree(element, name_='period', mapping_=mapping_)
        if self.modified is not None:
            modified_ = self.modified
            modified_.to_etree(element, name_='modified', mapping_=mapping_)
        for concern_ in self.concern:
            concern_.to_etree(element, name_='concern', mapping_=mapping_)
        for participant_ in self.participant:
            participant_.to_etree(element, name_='participant', mapping_=mapping_)
        for goal_ in self.goal:
            goal_.to_etree(element, name_='goal', mapping_=mapping_)
        for activity_ in self.activity:
            activity_.to_etree(element, name_='activity', mapping_=mapping_)
        if self.notes is not None:
            notes_ = self.notes
            notes_.to_etree(element, name_='notes', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(CarePlan, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'patient':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.patient = obj_
            obj_.original_tagname_ = 'patient'
        elif nodeName_ == 'status':
            obj_ = CarePlanStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'period':
            obj_ = Period.factory()
            obj_.build(child_)
            self.period = obj_
            obj_.original_tagname_ = 'period'
        elif nodeName_ == 'modified':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.modified = obj_
            obj_.original_tagname_ = 'modified'
        elif nodeName_ == 'concern':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.concern.append(obj_)
            obj_.original_tagname_ = 'concern'
        elif nodeName_ == 'participant':
            obj_ = CarePlan_Participant.factory()
            obj_.build(child_)
            self.participant.append(obj_)
            obj_.original_tagname_ = 'participant'
        elif nodeName_ == 'goal':
            obj_ = CarePlan_Goal.factory()
            obj_.build(child_)
            self.goal.append(obj_)
            obj_.original_tagname_ = 'goal'
        elif nodeName_ == 'activity':
            obj_ = CarePlan_Activity.factory()
            obj_.build(child_)
            self.activity.append(obj_)
            obj_.original_tagname_ = 'activity'
        elif nodeName_ == 'notes':
            obj_ = string.factory()
            obj_.build(child_)
            self.notes = obj_
            obj_.original_tagname_ = 'notes'
        super(CarePlan, self).buildChildren(child_, node, nodeName_, True)
# end class CarePlan


class CarePlan_Participant(BackboneElement):
    """Describes the intention of how one or more practitioners intend to
    deliver care for a particular patient for a period of time,
    possibly limited to care for a specific condition or set of
    conditions."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, role=None, member=None):
        self.original_tagname_ = None
        super(CarePlan_Participant, self).__init__(id, extension, modifierExtension, )
        self.role = role
        self.member = member
    def factory(*args_, **kwargs_):
        if CarePlan_Participant.subclass:
            return CarePlan_Participant.subclass(*args_, **kwargs_)
        else:
            return CarePlan_Participant(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_role(self): return self.role
    def set_role(self, role): self.role = role
    def get_member(self): return self.member
    def set_member(self, member): self.member = member
    def hasContent_(self):
        if (
            self.role is not None or
            self.member is not None or
            super(CarePlan_Participant, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlan.Participant', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Participant')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlan.Participant', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlan.Participant'):
        super(CarePlan_Participant, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Participant')
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlan.Participant', fromsubclass_=False, pretty_print=True):
        super(CarePlan_Participant, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.role is not None:
            self.role.export(outfile, level, namespace_, name_='role', pretty_print=pretty_print)
        if self.member is not None:
            self.member.export(outfile, level, namespace_, name_='member', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlan.Participant', mapping_=None):
        element = super(CarePlan_Participant, self).to_etree(parent_element, name_, mapping_)
        if self.role is not None:
            role_ = self.role
            role_.to_etree(element, name_='role', mapping_=mapping_)
        if self.member is not None:
            member_ = self.member
            member_.to_etree(element, name_='member', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(CarePlan_Participant, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'role':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.role = obj_
            obj_.original_tagname_ = 'role'
        elif nodeName_ == 'member':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.member = obj_
            obj_.original_tagname_ = 'member'
        super(CarePlan_Participant, self).buildChildren(child_, node, nodeName_, True)
# end class CarePlan_Participant


class CarePlan_Goal(BackboneElement):
    """Describes the intention of how one or more practitioners intend to
    deliver care for a particular patient for a period of time,
    possibly limited to care for a specific condition or set of
    conditions."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, description=None, status=None, notes=None, concern=None):
        self.original_tagname_ = None
        super(CarePlan_Goal, self).__init__(id, extension, modifierExtension, )
        self.description = description
        self.status = status
        self.notes = notes
        if concern is None:
            self.concern = []
        else:
            self.concern = concern
    def factory(*args_, **kwargs_):
        if CarePlan_Goal.subclass:
            return CarePlan_Goal.subclass(*args_, **kwargs_)
        else:
            return CarePlan_Goal(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_notes(self): return self.notes
    def set_notes(self, notes): self.notes = notes
    def get_concern(self): return self.concern
    def set_concern(self, concern): self.concern = concern
    def add_concern(self, value): self.concern.append(value)
    def insert_concern(self, index, value): self.concern[index] = value
    def hasContent_(self):
        if (
            self.description is not None or
            self.status is not None or
            self.notes is not None or
            self.concern or
            super(CarePlan_Goal, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlan.Goal', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Goal')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlan.Goal', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlan.Goal'):
        super(CarePlan_Goal, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Goal')
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlan.Goal', fromsubclass_=False, pretty_print=True):
        super(CarePlan_Goal, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.notes is not None:
            self.notes.export(outfile, level, namespace_, name_='notes', pretty_print=pretty_print)
        for concern_ in self.concern:
            concern_.export(outfile, level, namespace_, name_='concern', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlan.Goal', mapping_=None):
        element = super(CarePlan_Goal, self).to_etree(parent_element, name_, mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.notes is not None:
            notes_ = self.notes
            notes_.to_etree(element, name_='notes', mapping_=mapping_)
        for concern_ in self.concern:
            concern_.to_etree(element, name_='concern', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(CarePlan_Goal, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'status':
            obj_ = CarePlanGoalStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'notes':
            obj_ = string.factory()
            obj_.build(child_)
            self.notes = obj_
            obj_.original_tagname_ = 'notes'
        elif nodeName_ == 'concern':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.concern.append(obj_)
            obj_.original_tagname_ = 'concern'
        super(CarePlan_Goal, self).buildChildren(child_, node, nodeName_, True)
# end class CarePlan_Goal


class CarePlan_Activity(BackboneElement):
    """Describes the intention of how one or more practitioners intend to
    deliver care for a particular patient for a period of time,
    possibly limited to care for a specific condition or set of
    conditions."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, goal=None, status=None, prohibited=None, actionResulting=None, notes=None, detail=None, simple=None):
        self.original_tagname_ = None
        super(CarePlan_Activity, self).__init__(id, extension, modifierExtension, )
        if goal is None:
            self.goal = []
        else:
            self.goal = goal
        self.status = status
        self.prohibited = prohibited
        if actionResulting is None:
            self.actionResulting = []
        else:
            self.actionResulting = actionResulting
        self.notes = notes
        self.detail = detail
        self.simple = simple
    def factory(*args_, **kwargs_):
        if CarePlan_Activity.subclass:
            return CarePlan_Activity.subclass(*args_, **kwargs_)
        else:
            return CarePlan_Activity(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_goal(self): return self.goal
    def set_goal(self, goal): self.goal = goal
    def add_goal(self, value): self.goal.append(value)
    def insert_goal(self, index, value): self.goal[index] = value
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_prohibited(self): return self.prohibited
    def set_prohibited(self, prohibited): self.prohibited = prohibited
    def get_actionResulting(self): return self.actionResulting
    def set_actionResulting(self, actionResulting): self.actionResulting = actionResulting
    def add_actionResulting(self, value): self.actionResulting.append(value)
    def insert_actionResulting(self, index, value): self.actionResulting[index] = value
    def get_notes(self): return self.notes
    def set_notes(self, notes): self.notes = notes
    def get_detail(self): return self.detail
    def set_detail(self, detail): self.detail = detail
    def get_simple(self): return self.simple
    def set_simple(self, simple): self.simple = simple
    def validate_xmlIdRef(self, value):
        # Validate type xmlIdRef, a restriction on id-primitive.
        pass
    def hasContent_(self):
        if (
            self.goal or
            self.status is not None or
            self.prohibited is not None or
            self.actionResulting or
            self.notes is not None or
            self.detail is not None or
            self.simple is not None or
            super(CarePlan_Activity, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlan.Activity', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Activity')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlan.Activity', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlan.Activity'):
        super(CarePlan_Activity, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Activity')
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlan.Activity', fromsubclass_=False, pretty_print=True):
        super(CarePlan_Activity, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for goal_ in self.goal:
            goal_.export(outfile, level, namespace_, name_='goal', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.prohibited is not None:
            self.prohibited.export(outfile, level, namespace_, name_='prohibited', pretty_print=pretty_print)
        for actionResulting_ in self.actionResulting:
            actionResulting_.export(outfile, level, namespace_, name_='actionResulting', pretty_print=pretty_print)
        if self.notes is not None:
            self.notes.export(outfile, level, namespace_, name_='notes', pretty_print=pretty_print)
        if self.detail is not None:
            self.detail.export(outfile, level, namespace_, name_='detail', pretty_print=pretty_print)
        if self.simple is not None:
            self.simple.export(outfile, level, namespace_, name_='simple', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlan.Activity', mapping_=None):
        element = super(CarePlan_Activity, self).to_etree(parent_element, name_, mapping_)
        for goal_ in self.goal:
            goal_.to_etree(element, name_='goal', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.prohibited is not None:
            prohibited_ = self.prohibited
            prohibited_.to_etree(element, name_='prohibited', mapping_=mapping_)
        for actionResulting_ in self.actionResulting:
            actionResulting_.to_etree(element, name_='actionResulting', mapping_=mapping_)
        if self.notes is not None:
            notes_ = self.notes
            notes_.to_etree(element, name_='notes', mapping_=mapping_)
        if self.detail is not None:
            detail_ = self.detail
            detail_.to_etree(element, name_='detail', mapping_=mapping_)
        if self.simple is not None:
            simple_ = self.simple
            simple_.to_etree(element, name_='simple', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(CarePlan_Activity, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'goal':
            obj_ = None
            self.goal.append(obj_)
            obj_.original_tagname_ = 'goal'
            self.validate_xmlIdRef(self.goal)    # validate type xmlIdRef
        elif nodeName_ == 'status':
            obj_ = CarePlanActivityStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'prohibited':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.prohibited = obj_
            obj_.original_tagname_ = 'prohibited'
        elif nodeName_ == 'actionResulting':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.actionResulting.append(obj_)
            obj_.original_tagname_ = 'actionResulting'
        elif nodeName_ == 'notes':
            obj_ = string.factory()
            obj_.build(child_)
            self.notes = obj_
            obj_.original_tagname_ = 'notes'
        elif nodeName_ == 'detail':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.detail = obj_
            obj_.original_tagname_ = 'detail'
        elif nodeName_ == 'simple':
            obj_ = CarePlan_Simple.factory()
            obj_.build(child_)
            self.simple = obj_
            obj_.original_tagname_ = 'simple'
        super(CarePlan_Activity, self).buildChildren(child_, node, nodeName_, True)
# end class CarePlan_Activity


class CarePlan_Simple(BackboneElement):
    """Describes the intention of how one or more practitioners intend to
    deliver care for a particular patient for a period of time,
    possibly limited to care for a specific condition or set of
    conditions.The period, timing or frequency upon which the
    described activity is to occur."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, category=None, code=None, timingSchedule=None, timingPeriod=None, timingString=None, location=None, performer=None, product=None, dailyAmount=None, quantity=None, details=None):
        self.original_tagname_ = None
        super(CarePlan_Simple, self).__init__(id, extension, modifierExtension, )
        self.category = category
        self.code = code
        self.timingSchedule = timingSchedule
        self.timingPeriod = timingPeriod
        self.timingString = timingString
        self.location = location
        if performer is None:
            self.performer = []
        else:
            self.performer = performer
        self.product = product
        self.dailyAmount = dailyAmount
        self.quantity = quantity
        self.details = details
    def factory(*args_, **kwargs_):
        if CarePlan_Simple.subclass:
            return CarePlan_Simple.subclass(*args_, **kwargs_)
        else:
            return CarePlan_Simple(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_category(self): return self.category
    def set_category(self, category): self.category = category
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_timingSchedule(self): return self.timingSchedule
    def set_timingSchedule(self, timingSchedule): self.timingSchedule = timingSchedule
    def get_timingPeriod(self): return self.timingPeriod
    def set_timingPeriod(self, timingPeriod): self.timingPeriod = timingPeriod
    def get_timingString(self): return self.timingString
    def set_timingString(self, timingString): self.timingString = timingString
    def get_location(self): return self.location
    def set_location(self, location): self.location = location
    def get_performer(self): return self.performer
    def set_performer(self, performer): self.performer = performer
    def add_performer(self, value): self.performer.append(value)
    def insert_performer(self, index, value): self.performer[index] = value
    def get_product(self): return self.product
    def set_product(self, product): self.product = product
    def get_dailyAmount(self): return self.dailyAmount
    def set_dailyAmount(self, dailyAmount): self.dailyAmount = dailyAmount
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_details(self): return self.details
    def set_details(self, details): self.details = details
    def hasContent_(self):
        if (
            self.category is not None or
            self.code is not None or
            self.timingSchedule is not None or
            self.timingPeriod is not None or
            self.timingString is not None or
            self.location is not None or
            self.performer or
            self.product is not None or
            self.dailyAmount is not None or
            self.quantity is not None or
            self.details is not None or
            super(CarePlan_Simple, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlan.Simple', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Simple')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlan.Simple', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlan.Simple'):
        super(CarePlan_Simple, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlan.Simple')
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlan.Simple', fromsubclass_=False, pretty_print=True):
        super(CarePlan_Simple, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.category is not None:
            self.category.export(outfile, level, namespace_, name_='category', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.timingSchedule is not None:
            self.timingSchedule.export(outfile, level, namespace_, name_='timingSchedule', pretty_print=pretty_print)
        if self.timingPeriod is not None:
            self.timingPeriod.export(outfile, level, namespace_, name_='timingPeriod', pretty_print=pretty_print)
        if self.timingString is not None:
            self.timingString.export(outfile, level, namespace_, name_='timingString', pretty_print=pretty_print)
        if self.location is not None:
            self.location.export(outfile, level, namespace_, name_='location', pretty_print=pretty_print)
        for performer_ in self.performer:
            performer_.export(outfile, level, namespace_, name_='performer', pretty_print=pretty_print)
        if self.product is not None:
            self.product.export(outfile, level, namespace_, name_='product', pretty_print=pretty_print)
        if self.dailyAmount is not None:
            self.dailyAmount.export(outfile, level, namespace_, name_='dailyAmount', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        if self.details is not None:
            self.details.export(outfile, level, namespace_, name_='details', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlan.Simple', mapping_=None):
        element = super(CarePlan_Simple, self).to_etree(parent_element, name_, mapping_)
        if self.category is not None:
            category_ = self.category
            category_.to_etree(element, name_='category', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.timingSchedule is not None:
            timingSchedule_ = self.timingSchedule
            timingSchedule_.to_etree(element, name_='timingSchedule', mapping_=mapping_)
        if self.timingPeriod is not None:
            timingPeriod_ = self.timingPeriod
            timingPeriod_.to_etree(element, name_='timingPeriod', mapping_=mapping_)
        if self.timingString is not None:
            timingString_ = self.timingString
            timingString_.to_etree(element, name_='timingString', mapping_=mapping_)
        if self.location is not None:
            location_ = self.location
            location_.to_etree(element, name_='location', mapping_=mapping_)
        for performer_ in self.performer:
            performer_.to_etree(element, name_='performer', mapping_=mapping_)
        if self.product is not None:
            product_ = self.product
            product_.to_etree(element, name_='product', mapping_=mapping_)
        if self.dailyAmount is not None:
            dailyAmount_ = self.dailyAmount
            dailyAmount_.to_etree(element, name_='dailyAmount', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if self.details is not None:
            details_ = self.details
            details_.to_etree(element, name_='details', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(CarePlan_Simple, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'category':
            obj_ = CarePlanActivityCategory.factory()
            obj_.build(child_)
            self.category = obj_
            obj_.original_tagname_ = 'category'
        elif nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'timingSchedule':
            obj_ = Schedule.factory()
            obj_.build(child_)
            self.timingSchedule = obj_
            obj_.original_tagname_ = 'timingSchedule'
        elif nodeName_ == 'timingPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.timingPeriod = obj_
            obj_.original_tagname_ = 'timingPeriod'
        elif nodeName_ == 'timingString':
            obj_ = string.factory()
            obj_.build(child_)
            self.timingString = obj_
            obj_.original_tagname_ = 'timingString'
        elif nodeName_ == 'location':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.location = obj_
            obj_.original_tagname_ = 'location'
        elif nodeName_ == 'performer':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.performer.append(obj_)
            obj_.original_tagname_ = 'performer'
        elif nodeName_ == 'product':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.product = obj_
            obj_.original_tagname_ = 'product'
        elif nodeName_ == 'dailyAmount':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.dailyAmount = obj_
            obj_.original_tagname_ = 'dailyAmount'
        elif nodeName_ == 'quantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'details':
            obj_ = string.factory()
            obj_.build(child_)
            self.details = obj_
            obj_.original_tagname_ = 'details'
        super(CarePlan_Simple, self).buildChildren(child_, node, nodeName_, True)
# end class CarePlan_Simple


class CarePlanStatus(Element):
    """Indicates whether the plan is currently being acted upon, represents
    future intentions or is now just historical record.If the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(CarePlanStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if CarePlanStatus.subclass:
            return CarePlanStatus.subclass(*args_, **kwargs_)
        else:
            return CarePlanStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(CarePlanStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlanStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlanStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlanStatus'):
        super(CarePlanStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlanStatus', fromsubclass_=False, pretty_print=True):
        super(CarePlanStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlanStatus', mapping_=None):
        element = super(CarePlanStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(CarePlanStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(CarePlanStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class CarePlanStatus


class CarePlanActivityCategory(Element):
    """High-level categorization of the type of activity in a care plan.If
    the element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(CarePlanActivityCategory, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if CarePlanActivityCategory.subclass:
            return CarePlanActivityCategory.subclass(*args_, **kwargs_)
        else:
            return CarePlanActivityCategory(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(CarePlanActivityCategory, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlanActivityCategory', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanActivityCategory')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlanActivityCategory', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlanActivityCategory'):
        super(CarePlanActivityCategory, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanActivityCategory')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlanActivityCategory', fromsubclass_=False, pretty_print=True):
        super(CarePlanActivityCategory, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlanActivityCategory', mapping_=None):
        element = super(CarePlanActivityCategory, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(CarePlanActivityCategory, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(CarePlanActivityCategory, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class CarePlanActivityCategory


class CarePlanGoalStatus(Element):
    """Indicates whether the goal has been met and is still being
    targetedIf the element is present, it must have either a @value,
    an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(CarePlanGoalStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if CarePlanGoalStatus.subclass:
            return CarePlanGoalStatus.subclass(*args_, **kwargs_)
        else:
            return CarePlanGoalStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(CarePlanGoalStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlanGoalStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanGoalStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlanGoalStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlanGoalStatus'):
        super(CarePlanGoalStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanGoalStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlanGoalStatus', fromsubclass_=False, pretty_print=True):
        super(CarePlanGoalStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlanGoalStatus', mapping_=None):
        element = super(CarePlanGoalStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(CarePlanGoalStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(CarePlanGoalStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class CarePlanGoalStatus


class CarePlanActivityStatus(Element):
    """Indicates where the activity is at in its overall life cycleIf the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(CarePlanActivityStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if CarePlanActivityStatus.subclass:
            return CarePlanActivityStatus.subclass(*args_, **kwargs_)
        else:
            return CarePlanActivityStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(CarePlanActivityStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CarePlanActivityStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanActivityStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CarePlanActivityStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CarePlanActivityStatus'):
        super(CarePlanActivityStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CarePlanActivityStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='CarePlanActivityStatus', fromsubclass_=False, pretty_print=True):
        super(CarePlanActivityStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CarePlanActivityStatus', mapping_=None):
        element = super(CarePlanActivityStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(CarePlanActivityStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(CarePlanActivityStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class CarePlanActivityStatus



from .base_classes import *
from .support_functions import *


class DeviceObservationReport(Resource):
    """Describes the data produced by a device at a point in time.If the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, instant=None, identifier=None, source=None, subject=None, virtualDevice=None):
        self.original_tagname_ = None
        super(DeviceObservationReport, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.instant = instant
        self.identifier = identifier
        self.source = source
        self.subject = subject
        if virtualDevice is None:
            self.virtualDevice = []
        else:
            self.virtualDevice = virtualDevice
    def factory(*args_, **kwargs_):
        if DeviceObservationReport.subclass:
            return DeviceObservationReport.subclass(*args_, **kwargs_)
        else:
            return DeviceObservationReport(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_instant(self): return self.instant
    def set_instant(self, instant): self.instant = instant
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_virtualDevice(self): return self.virtualDevice
    def set_virtualDevice(self, virtualDevice): self.virtualDevice = virtualDevice
    def add_virtualDevice(self, value): self.virtualDevice.append(value)
    def insert_virtualDevice(self, index, value): self.virtualDevice[index] = value
    def hasContent_(self):
        if (
            self.instant is not None or
            self.identifier is not None or
            self.source is not None or
            self.subject is not None or
            self.virtualDevice or
            super(DeviceObservationReport, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DeviceObservationReport', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DeviceObservationReport', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DeviceObservationReport'):
        super(DeviceObservationReport, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport')
    def exportChildren(self, outfile, level, namespace_='', name_='DeviceObservationReport', fromsubclass_=False, pretty_print=True):
        super(DeviceObservationReport, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.instant is not None:
            self.instant.export(outfile, level, namespace_, name_='instant', pretty_print=pretty_print)
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.source is not None:
            self.source.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        for virtualDevice_ in self.virtualDevice:
            virtualDevice_.export(outfile, level, namespace_, name_='virtualDevice', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DeviceObservationReport', mapping_=None):
        element = super(DeviceObservationReport, self).to_etree(parent_element, name_, mapping_)
        if self.instant is not None:
            instant_ = self.instant
            instant_.to_etree(element, name_='instant', mapping_=mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.source is not None:
            source_ = self.source
            source_.to_etree(element, name_='source', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        for virtualDevice_ in self.virtualDevice:
            virtualDevice_.to_etree(element, name_='virtualDevice', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DeviceObservationReport, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'instant':
            obj_ = instant.factory()
            obj_.build(child_)
            self.instant = obj_
            obj_.original_tagname_ = 'instant'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'source':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.source = obj_
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'virtualDevice':
            obj_ = DeviceObservationReport_VirtualDevice.factory()
            obj_.build(child_)
            self.virtualDevice.append(obj_)
            obj_.original_tagname_ = 'virtualDevice'
        super(DeviceObservationReport, self).buildChildren(child_, node, nodeName_, True)
# end class DeviceObservationReport


class DeviceObservationReport_VirtualDevice(BackboneElement):
    """Describes the data produced by a device at a point in time."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, channel=None):
        self.original_tagname_ = None
        super(DeviceObservationReport_VirtualDevice, self).__init__(id, extension, modifierExtension, )
        self.code = code
        if channel is None:
            self.channel = []
        else:
            self.channel = channel
    def factory(*args_, **kwargs_):
        if DeviceObservationReport_VirtualDevice.subclass:
            return DeviceObservationReport_VirtualDevice.subclass(*args_, **kwargs_)
        else:
            return DeviceObservationReport_VirtualDevice(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_channel(self): return self.channel
    def set_channel(self, channel): self.channel = channel
    def add_channel(self, value): self.channel.append(value)
    def insert_channel(self, index, value): self.channel[index] = value
    def hasContent_(self):
        if (
            self.code is not None or
            self.channel or
            super(DeviceObservationReport_VirtualDevice, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DeviceObservationReport.VirtualDevice', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport.VirtualDevice')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DeviceObservationReport.VirtualDevice', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DeviceObservationReport.VirtualDevice'):
        super(DeviceObservationReport_VirtualDevice, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport.VirtualDevice')
    def exportChildren(self, outfile, level, namespace_='', name_='DeviceObservationReport.VirtualDevice', fromsubclass_=False, pretty_print=True):
        super(DeviceObservationReport_VirtualDevice, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        for channel_ in self.channel:
            channel_.export(outfile, level, namespace_, name_='channel', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DeviceObservationReport.VirtualDevice', mapping_=None):
        element = super(DeviceObservationReport_VirtualDevice, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        for channel_ in self.channel:
            channel_.to_etree(element, name_='channel', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DeviceObservationReport_VirtualDevice, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'channel':
            obj_ = DeviceObservationReport_Channel.factory()
            obj_.build(child_)
            self.channel.append(obj_)
            obj_.original_tagname_ = 'channel'
        super(DeviceObservationReport_VirtualDevice, self).buildChildren(child_, node, nodeName_, True)
# end class DeviceObservationReport_VirtualDevice


class DeviceObservationReport_Channel(BackboneElement):
    """Describes the data produced by a device at a point in time."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, metric=None):
        self.original_tagname_ = None
        super(DeviceObservationReport_Channel, self).__init__(id, extension, modifierExtension, )
        self.code = code
        if metric is None:
            self.metric = []
        else:
            self.metric = metric
    def factory(*args_, **kwargs_):
        if DeviceObservationReport_Channel.subclass:
            return DeviceObservationReport_Channel.subclass(*args_, **kwargs_)
        else:
            return DeviceObservationReport_Channel(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_metric(self): return self.metric
    def set_metric(self, metric): self.metric = metric
    def add_metric(self, value): self.metric.append(value)
    def insert_metric(self, index, value): self.metric[index] = value
    def hasContent_(self):
        if (
            self.code is not None or
            self.metric or
            super(DeviceObservationReport_Channel, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DeviceObservationReport.Channel', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport.Channel')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DeviceObservationReport.Channel', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DeviceObservationReport.Channel'):
        super(DeviceObservationReport_Channel, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport.Channel')
    def exportChildren(self, outfile, level, namespace_='', name_='DeviceObservationReport.Channel', fromsubclass_=False, pretty_print=True):
        super(DeviceObservationReport_Channel, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        for metric_ in self.metric:
            metric_.export(outfile, level, namespace_, name_='metric', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DeviceObservationReport.Channel', mapping_=None):
        element = super(DeviceObservationReport_Channel, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        for metric_ in self.metric:
            metric_.to_etree(element, name_='metric', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DeviceObservationReport_Channel, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'metric':
            obj_ = DeviceObservationReport_Metric.factory()
            obj_.build(child_)
            self.metric.append(obj_)
            obj_.original_tagname_ = 'metric'
        super(DeviceObservationReport_Channel, self).buildChildren(child_, node, nodeName_, True)
# end class DeviceObservationReport_Channel


class DeviceObservationReport_Metric(BackboneElement):
    """Describes the data produced by a device at a point in time."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, observation=None):
        self.original_tagname_ = None
        super(DeviceObservationReport_Metric, self).__init__(id, extension, modifierExtension, )
        self.observation = observation
    def factory(*args_, **kwargs_):
        if DeviceObservationReport_Metric.subclass:
            return DeviceObservationReport_Metric.subclass(*args_, **kwargs_)
        else:
            return DeviceObservationReport_Metric(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_observation(self): return self.observation
    def set_observation(self, observation): self.observation = observation
    def hasContent_(self):
        if (
            self.observation is not None or
            super(DeviceObservationReport_Metric, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DeviceObservationReport.Metric', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport.Metric')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DeviceObservationReport.Metric', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DeviceObservationReport.Metric'):
        super(DeviceObservationReport_Metric, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DeviceObservationReport.Metric')
    def exportChildren(self, outfile, level, namespace_='', name_='DeviceObservationReport.Metric', fromsubclass_=False, pretty_print=True):
        super(DeviceObservationReport_Metric, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.observation is not None:
            self.observation.export(outfile, level, namespace_, name_='observation', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DeviceObservationReport.Metric', mapping_=None):
        element = super(DeviceObservationReport_Metric, self).to_etree(parent_element, name_, mapping_)
        if self.observation is not None:
            observation_ = self.observation
            observation_.to_etree(element, name_='observation', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DeviceObservationReport_Metric, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'observation':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.observation = obj_
            obj_.original_tagname_ = 'observation'
        super(DeviceObservationReport_Metric, self).buildChildren(child_, node, nodeName_, True)
# end class DeviceObservationReport_Metric



from StringIO import StringIO
from operator import attrgetter
from .datastore import find_record
from server.common import safe_attrgetter
from .health_mixin import ExportXMLMixin
import server.fhir as supermod
import sys

try:
    from flask import current_app, url_for
    RUN_FLASK=True
except:
    from .datastore import dumb_url_generate
    RUN_FLASK=False

class Practitioner_Map:

    #No requirements yet
    root_search=[]

    #Just use row id
    url_prefixes={}
            #'gnuhealth.healthprofessional': ''}

    model_mapping={
            'gnuhealth.healthprofessional': {
                'communication': 'name.lang',
                'specialty': 'specialties',
                'role': 'name.occupation.name',
                'gender': 'name.sex',
                'identifier': 'name.puid',
                'name': 'name.person_names'}}

    resource_search_params={
                    '_id': 'token',
                    '_language': None,
                    'address': None,
                    'family': 'string',
                    'gender': 'token',
                    'given': 'string',
                    'identifier': 'token',
                    'name': 'string',
                    'organization': None,
                    'phonetic': None,
                    'telecom': None}
    search_mapping={
                '_id': ['id'],
                    'family': ['name.lastname'],
                    'gender': ['name.sex'],
                    'given': ['name.name'],
                    'identifier': ['name.puid'],
                    'name': ['name.lastname', 'name.name']}

class health_Practitioner(supermod.Practitioner, Practitioner_Map, ExportXMLMixin):
    def __init__(self, *args, **kwargs):
        rec = kwargs.pop('gnu_record', None)
        super(health_Practitioner, self).__init__(*args, **kwargs)
        if rec:
            self.set_gnu_practitioner(rec)

    def set_gnu_practitioner(self, practitioner):
        """Set the GNU Health record
        ::::
            params:
                practitioner ===> Health model
            returns:
                instance
        """
        self.practitioner = practitioner
        self.model_type = self.practitioner.__name__

        # Only certain models
        if self.model_type not in self.model_mapping:
            raise ValueError('Not a valid model')

        self.map = self.model_mapping[self.model_type]
        #self.search_prefix=self.url_prefixes[self.model_type]

        self.__import_from_gnu_practitioner()
        self.__set_feed_info()

    def __import_from_gnu_practitioner(self):
        """Set data from model"""
        if self.practitioner:
            self.set_specialty(safe_attrgetter(self.practitioner, self.map['specialty']))
            self.set_role(safe_attrgetter(self.practitioner, self.map['role']))
            self.set_communication(safe_attrgetter(self.practitioner, self.map['communication']))
            self.set_gender(safe_attrgetter(self.practitioner, self.map['gender']))
            self.set_name(safe_attrgetter(self.practitioner, self.map['name']))
            self.set_identifier(safe_attrgetter(self.practitioner, self.map['identifier']))

    def __set_feed_info(self):
        """Set the feed-relevant data"""
        if self.practitioner:
            if RUN_FLASK:
                uri = url_for('hp_record',
                                log_id=self.practitioner.id,
                                _external=True)
            else:
                uri = dumb_url_generate(['Practitioner',
                                self.practitioner.id])
            self.feed={'id': uri,
                    'published': self.practitioner.name.create_date,
                    'updated': self.practitioner.name.write_date or self.practitioner.name.create_date,
                    'title': self.practitioner.name.rec_name
                        }

    def set_name(self, names):
        """Extends superclass for convenience

        Keyword arguments:
        names -- names from model
        """

        for name in names:
            if name.use in ('official', 'usual'):
                given=[supermod.string(value=x) for x in name.given.split()]
                family=[supermod.string(value=x) for x in name.family.split()]
                super(health_Practitioner, self).set_name(
                            supermod.HumanName(
                            use=supermod.NameUse(value=name.use),
                            family=family,
                            given=given))
                return


    def set_identifier(self, identifier):
        """Extends superclass for convenience

        Keyword arguments:
        identifier -- puid
        """

        if identifier:
            ident = supermod.Identifier(
                        use=supermod.IdentifierUse(value='usual'),
                        label=supermod.string(value='PUID'),
                        value=supermod.string(value=identifier))
            super(health_Practitioner, self).add_identifier(ident)

    def set_gender(self, gender):
        """Extends superclass for convenience

        Keyword arguments:
        gender -- practitioner's gender
        """

        if gender:
            from server.fhir.value_sets import administrativeGender as codes
            us = gender.upper()
            sd = [x for x in codes.contents if x['code'] == us][0]
            coding = supermod.Coding(
                        system=supermod.uri(value=sd['system']),
                        code=supermod.code(value=sd['code']),
                        display=supermod.string(value=sd['display'])
                        )
            g=supermod.CodeableConcept(coding=[coding])
            super(health_Practitioner, self).set_gender(g)

    def set_communication(self, communication):
        """Extends superclass for convenience

        Keyword arguments:
        communication -- language
        """

        if communication:
            from re import sub
            code=sub('_','-', communication.code)
            name=communication.name
            coding = supermod.Coding(
                        system=supermod.uri(value='urn:ietf:bcp:47'),
                        code=supermod.code(value=code),
                        display=supermod.string(value=name)
                        )
            com=supermod.CodeableConcept(coding=[coding],
                                    text=supermod.string(value=name))
            super(health_Practitioner, self).add_communication(com)

    def set_specialty(self, specialty):
        """Extends superclass for convenience

        Keyword arguments
        specialty -- person's specialties
        """

        for spec in  specialty:
            code, name = attrgetter('specialty.code', 'specialty.name')(spec)
            coding = supermod.Coding(code=supermod.string(value=code),
                    display=supermod.string(value=name))
            com=supermod.CodeableConcept(coding=[coding])
            super(health_Practitioner, self).add_specialty(com)

    def set_role(self, role):
        """Extends superclass for convenience

        Keyword arguments:
        role -- occupation name
        """

        if role:
            coding = supermod.Coding(display=supermod.string(value=str(role)))
            com=supermod.CodeableConcept(coding=[coding])
            super(health_Practitioner, self).set_role([com])

supermod.Practitioner.subclass=health_Practitioner



from .base_classes import *
from .support_functions import *


class ConceptMap(Resource):
    """A statement of relationships from one set of concepts to one or more
    other concept systems.If the element is present, it must have
    either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, version=None, name=None, publisher=None, telecom=None, description=None, copyright=None, status=None, experimental=None, date=None, source=None, target=None, concept=None):
        self.original_tagname_ = None
        super(ConceptMap, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.identifier = identifier
        self.version = version
        self.name = name
        self.publisher = publisher
        if telecom is None:
            self.telecom = []
        else:
            self.telecom = telecom
        self.description = description
        self.copyright = copyright
        self.status = status
        self.experimental = experimental
        self.date = date
        self.source = source
        self.target = target
        if concept is None:
            self.concept = []
        else:
            self.concept = concept
    def factory(*args_, **kwargs_):
        if ConceptMap.subclass:
            return ConceptMap.subclass(*args_, **kwargs_)
        else:
            return ConceptMap(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_version(self): return self.version
    def set_version(self, version): self.version = version
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_publisher(self): return self.publisher
    def set_publisher(self, publisher): self.publisher = publisher
    def get_telecom(self): return self.telecom
    def set_telecom(self, telecom): self.telecom = telecom
    def add_telecom(self, value): self.telecom.append(value)
    def insert_telecom(self, index, value): self.telecom[index] = value
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_copyright(self): return self.copyright
    def set_copyright(self, copyright): self.copyright = copyright
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_experimental(self): return self.experimental
    def set_experimental(self, experimental): self.experimental = experimental
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def get_target(self): return self.target
    def set_target(self, target): self.target = target
    def get_concept(self): return self.concept
    def set_concept(self, concept): self.concept = concept
    def add_concept(self, value): self.concept.append(value)
    def insert_concept(self, index, value): self.concept[index] = value
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.version is not None or
            self.name is not None or
            self.publisher is not None or
            self.telecom or
            self.description is not None or
            self.copyright is not None or
            self.status is not None or
            self.experimental is not None or
            self.date is not None or
            self.source is not None or
            self.target is not None or
            self.concept or
            super(ConceptMap, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ConceptMap', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ConceptMap', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ConceptMap'):
        super(ConceptMap, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap')
    def exportChildren(self, outfile, level, namespace_='', name_='ConceptMap', fromsubclass_=False, pretty_print=True):
        super(ConceptMap, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.version is not None:
            self.version.export(outfile, level, namespace_, name_='version', pretty_print=pretty_print)
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.publisher is not None:
            self.publisher.export(outfile, level, namespace_, name_='publisher', pretty_print=pretty_print)
        for telecom_ in self.telecom:
            telecom_.export(outfile, level, namespace_, name_='telecom', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.copyright is not None:
            self.copyright.export(outfile, level, namespace_, name_='copyright', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.experimental is not None:
            self.experimental.export(outfile, level, namespace_, name_='experimental', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.source is not None:
            self.source.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        if self.target is not None:
            self.target.export(outfile, level, namespace_, name_='target', pretty_print=pretty_print)
        for concept_ in self.concept:
            concept_.export(outfile, level, namespace_, name_='concept', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ConceptMap', mapping_=None):
        element = super(ConceptMap, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.version is not None:
            version_ = self.version
            version_.to_etree(element, name_='version', mapping_=mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.publisher is not None:
            publisher_ = self.publisher
            publisher_.to_etree(element, name_='publisher', mapping_=mapping_)
        for telecom_ in self.telecom:
            telecom_.to_etree(element, name_='telecom', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.copyright is not None:
            copyright_ = self.copyright
            copyright_.to_etree(element, name_='copyright', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.experimental is not None:
            experimental_ = self.experimental
            experimental_.to_etree(element, name_='experimental', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.source is not None:
            source_ = self.source
            source_.to_etree(element, name_='source', mapping_=mapping_)
        if self.target is not None:
            target_ = self.target
            target_.to_etree(element, name_='target', mapping_=mapping_)
        for concept_ in self.concept:
            concept_.to_etree(element, name_='concept', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ConceptMap, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = string.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'version':
            obj_ = string.factory()
            obj_.build(child_)
            self.version = obj_
            obj_.original_tagname_ = 'version'
        elif nodeName_ == 'name':
            obj_ = string.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'publisher':
            obj_ = string.factory()
            obj_.build(child_)
            self.publisher = obj_
            obj_.original_tagname_ = 'publisher'
        elif nodeName_ == 'telecom':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.telecom.append(obj_)
            obj_.original_tagname_ = 'telecom'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'copyright':
            obj_ = string.factory()
            obj_.build(child_)
            self.copyright = obj_
            obj_.original_tagname_ = 'copyright'
        elif nodeName_ == 'status':
            obj_ = ValueSetStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'experimental':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.experimental = obj_
            obj_.original_tagname_ = 'experimental'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'source':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.source = obj_
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'target':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.target = obj_
            obj_.original_tagname_ = 'target'
        elif nodeName_ == 'concept':
            obj_ = ConceptMap_Concept.factory()
            obj_.build(child_)
            self.concept.append(obj_)
            obj_.original_tagname_ = 'concept'
        super(ConceptMap, self).buildChildren(child_, node, nodeName_, True)
# end class ConceptMap


class ConceptMap_Concept(BackboneElement):
    """A statement of relationships from one set of concepts to one or more
    other concept systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, system=None, code=None, dependsOn=None, map=None):
        self.original_tagname_ = None
        super(ConceptMap_Concept, self).__init__(id, extension, modifierExtension, )
        self.system = system
        self.code = code
        if dependsOn is None:
            self.dependsOn = []
        else:
            self.dependsOn = dependsOn
        if map is None:
            self.map = []
        else:
            self.map = map
    def factory(*args_, **kwargs_):
        if ConceptMap_Concept.subclass:
            return ConceptMap_Concept.subclass(*args_, **kwargs_)
        else:
            return ConceptMap_Concept(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_system(self): return self.system
    def set_system(self, system): self.system = system
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_dependsOn(self): return self.dependsOn
    def set_dependsOn(self, dependsOn): self.dependsOn = dependsOn
    def add_dependsOn(self, value): self.dependsOn.append(value)
    def insert_dependsOn(self, index, value): self.dependsOn[index] = value
    def get_map(self): return self.map
    def set_map(self, map): self.map = map
    def add_map(self, value): self.map.append(value)
    def insert_map(self, index, value): self.map[index] = value
    def hasContent_(self):
        if (
            self.system is not None or
            self.code is not None or
            self.dependsOn or
            self.map or
            super(ConceptMap_Concept, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ConceptMap.Concept', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap.Concept')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ConceptMap.Concept', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ConceptMap.Concept'):
        super(ConceptMap_Concept, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap.Concept')
    def exportChildren(self, outfile, level, namespace_='', name_='ConceptMap.Concept', fromsubclass_=False, pretty_print=True):
        super(ConceptMap_Concept, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.system is not None:
            self.system.export(outfile, level, namespace_, name_='system', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        for dependsOn_ in self.dependsOn:
            dependsOn_.export(outfile, level, namespace_, name_='dependsOn', pretty_print=pretty_print)
        for map_ in self.map:
            map_.export(outfile, level, namespace_, name_='map', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ConceptMap.Concept', mapping_=None):
        element = super(ConceptMap_Concept, self).to_etree(parent_element, name_, mapping_)
        if self.system is not None:
            system_ = self.system
            system_.to_etree(element, name_='system', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        for dependsOn_ in self.dependsOn:
            dependsOn_.to_etree(element, name_='dependsOn', mapping_=mapping_)
        for map_ in self.map:
            map_.to_etree(element, name_='map', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ConceptMap_Concept, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'system':
            obj_ = uri.factory()
            obj_.build(child_)
            self.system = obj_
            obj_.original_tagname_ = 'system'
        elif nodeName_ == 'code':
            obj_ = code.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'dependsOn':
            obj_ = ConceptMap_DependsOn.factory()
            obj_.build(child_)
            self.dependsOn.append(obj_)
            obj_.original_tagname_ = 'dependsOn'
        elif nodeName_ == 'map':
            obj_ = ConceptMap_Map.factory()
            obj_.build(child_)
            self.map.append(obj_)
            obj_.original_tagname_ = 'map'
        super(ConceptMap_Concept, self).buildChildren(child_, node, nodeName_, True)
# end class ConceptMap_Concept


class ConceptMap_DependsOn(BackboneElement):
    """A statement of relationships from one set of concepts to one or more
    other concept systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, concept=None, system=None, code=None):
        self.original_tagname_ = None
        super(ConceptMap_DependsOn, self).__init__(id, extension, modifierExtension, )
        self.concept = concept
        self.system = system
        self.code = code
    def factory(*args_, **kwargs_):
        if ConceptMap_DependsOn.subclass:
            return ConceptMap_DependsOn.subclass(*args_, **kwargs_)
        else:
            return ConceptMap_DependsOn(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_concept(self): return self.concept
    def set_concept(self, concept): self.concept = concept
    def get_system(self): return self.system
    def set_system(self, system): self.system = system
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def hasContent_(self):
        if (
            self.concept is not None or
            self.system is not None or
            self.code is not None or
            super(ConceptMap_DependsOn, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ConceptMap.DependsOn', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap.DependsOn')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ConceptMap.DependsOn', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ConceptMap.DependsOn'):
        super(ConceptMap_DependsOn, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap.DependsOn')
    def exportChildren(self, outfile, level, namespace_='', name_='ConceptMap.DependsOn', fromsubclass_=False, pretty_print=True):
        super(ConceptMap_DependsOn, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.concept is not None:
            self.concept.export(outfile, level, namespace_, name_='concept', pretty_print=pretty_print)
        if self.system is not None:
            self.system.export(outfile, level, namespace_, name_='system', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ConceptMap.DependsOn', mapping_=None):
        element = super(ConceptMap_DependsOn, self).to_etree(parent_element, name_, mapping_)
        if self.concept is not None:
            concept_ = self.concept
            concept_.to_etree(element, name_='concept', mapping_=mapping_)
        if self.system is not None:
            system_ = self.system
            system_.to_etree(element, name_='system', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ConceptMap_DependsOn, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'concept':
            obj_ = uri.factory()
            obj_.build(child_)
            self.concept = obj_
            obj_.original_tagname_ = 'concept'
        elif nodeName_ == 'system':
            obj_ = uri.factory()
            obj_.build(child_)
            self.system = obj_
            obj_.original_tagname_ = 'system'
        elif nodeName_ == 'code':
            obj_ = code.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        super(ConceptMap_DependsOn, self).buildChildren(child_, node, nodeName_, True)
# end class ConceptMap_DependsOn


class ConceptMap_Map(BackboneElement):
    """A statement of relationships from one set of concepts to one or more
    other concept systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, system=None, code=None, equivalence=None, comments=None, product=None):
        self.original_tagname_ = None
        super(ConceptMap_Map, self).__init__(id, extension, modifierExtension, )
        self.system = system
        self.code = code
        self.equivalence = equivalence
        self.comments = comments
        if product is None:
            self.product = []
        else:
            self.product = product
    def factory(*args_, **kwargs_):
        if ConceptMap_Map.subclass:
            return ConceptMap_Map.subclass(*args_, **kwargs_)
        else:
            return ConceptMap_Map(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_system(self): return self.system
    def set_system(self, system): self.system = system
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_equivalence(self): return self.equivalence
    def set_equivalence(self, equivalence): self.equivalence = equivalence
    def get_comments(self): return self.comments
    def set_comments(self, comments): self.comments = comments
    def get_product(self): return self.product
    def set_product(self, product): self.product = product
    def add_product(self, value): self.product.append(value)
    def insert_product(self, index, value): self.product[index] = value
    def hasContent_(self):
        if (
            self.system is not None or
            self.code is not None or
            self.equivalence is not None or
            self.comments is not None or
            self.product or
            super(ConceptMap_Map, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ConceptMap.Map', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap.Map')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ConceptMap.Map', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ConceptMap.Map'):
        super(ConceptMap_Map, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMap.Map')
    def exportChildren(self, outfile, level, namespace_='', name_='ConceptMap.Map', fromsubclass_=False, pretty_print=True):
        super(ConceptMap_Map, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.system is not None:
            self.system.export(outfile, level, namespace_, name_='system', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.equivalence is not None:
            self.equivalence.export(outfile, level, namespace_, name_='equivalence', pretty_print=pretty_print)
        if self.comments is not None:
            self.comments.export(outfile, level, namespace_, name_='comments', pretty_print=pretty_print)
        for product_ in self.product:
            product_.export(outfile, level, namespace_, name_='product', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ConceptMap.Map', mapping_=None):
        element = super(ConceptMap_Map, self).to_etree(parent_element, name_, mapping_)
        if self.system is not None:
            system_ = self.system
            system_.to_etree(element, name_='system', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.equivalence is not None:
            equivalence_ = self.equivalence
            equivalence_.to_etree(element, name_='equivalence', mapping_=mapping_)
        if self.comments is not None:
            comments_ = self.comments
            comments_.to_etree(element, name_='comments', mapping_=mapping_)
        for product_ in self.product:
            product_.to_etree(element, name_='product', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ConceptMap_Map, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'system':
            obj_ = uri.factory()
            obj_.build(child_)
            self.system = obj_
            obj_.original_tagname_ = 'system'
        elif nodeName_ == 'code':
            obj_ = code.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'equivalence':
            obj_ = ConceptMapEquivalence.factory()
            obj_.build(child_)
            self.equivalence = obj_
            obj_.original_tagname_ = 'equivalence'
        elif nodeName_ == 'comments':
            obj_ = string.factory()
            obj_.build(child_)
            self.comments = obj_
            obj_.original_tagname_ = 'comments'
        elif nodeName_ == 'product':
            obj_ = ConceptMap_DependsOn.factory()
            obj_.build(child_)
            self.product.append(obj_)
            obj_.original_tagname_ = 'product'
        super(ConceptMap_Map, self).buildChildren(child_, node, nodeName_, True)
# end class ConceptMap_Map


class ConceptMapEquivalence(Element):
    """The degree of equivalence between conceptsIf the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ConceptMapEquivalence, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ConceptMapEquivalence.subclass:
            return ConceptMapEquivalence.subclass(*args_, **kwargs_)
        else:
            return ConceptMapEquivalence(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ConceptMapEquivalence, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ConceptMapEquivalence', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMapEquivalence')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ConceptMapEquivalence', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ConceptMapEquivalence'):
        super(ConceptMapEquivalence, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ConceptMapEquivalence')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ConceptMapEquivalence', fromsubclass_=False, pretty_print=True):
        super(ConceptMapEquivalence, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ConceptMapEquivalence', mapping_=None):
        element = super(ConceptMapEquivalence, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ConceptMapEquivalence, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ConceptMapEquivalence, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ConceptMapEquivalence


class ValueSetStatus(Element):
    """If the element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ValueSetStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ValueSetStatus.subclass:
            return ValueSetStatus.subclass(*args_, **kwargs_)
        else:
            return ValueSetStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ValueSetStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSetStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSetStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSetStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSetStatus'):
        super(ValueSetStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSetStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSetStatus', fromsubclass_=False, pretty_print=True):
        super(ValueSetStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSetStatus', mapping_=None):
        element = super(ValueSetStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ValueSetStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ValueSetStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ValueSetStatus



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, ModelSingleton, ModelSQL, fields, Unique
from trytond.transaction import Transaction
from trytond.pool import Pool


__all__ = ['GnuHealthSequences', 'PatientData', 'TestType', 'Lab',
    'GnuHealthLabTestUnits', 'GnuHealthTestCritearea',
    'GnuHealthPatientLabTest']


class GnuHealthSequences(ModelSingleton, ModelSQL, ModelView):
    "Standard Sequences for GNU Health"
    __name__ = "gnuhealth.sequences"

    lab_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Lab Sequence', domain=[('code', '=', 'gnuhealth.lab')],
        required=True))
    lab_request_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Patient Lab Request Sequence', required=True,
        domain=[('code', '=', 'gnuhealth.patient.lab.test')]))


class PatientData(ModelSQL, ModelView):
    'Patient lab tests'
    __name__ = 'gnuhealth.patient'

    lab_test_ids = fields.One2Many('gnuhealth.patient.lab.test', 'patient_id',
        'Lab Tests Required')


class TestType(ModelSQL, ModelView):
    'Type of Lab test'
    __name__ = 'gnuhealth.lab.test_type'

    name = fields.Char('Test',
        help="Test type, eg X-Ray, hemogram,biopsy...", required=True,
        select=True, translate=True)
    code = fields.Char('Code',
        help="Short name - code for the test", required=True, select=True)
    info = fields.Text('Description')
    product_id = fields.Many2One('product.product', 'Service', required=True)
    critearea = fields.One2Many('gnuhealth.lab.test.critearea', 'test_type_id',
        'Test Cases')

    active = fields.Boolean('Active', select=True)

    @staticmethod
    def default_active():
        return True

    @classmethod
    def __setup__(cls):
        super(TestType, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t, t.name),
             'The Lab Test code must be unique')
        ]

    @classmethod
    def check_xml_record(cls, records, values):
        return True

    @classmethod
    def search_rec_name(cls, name, clause):
        """ Search for the full name and the code """
        field = None
        for field in ('name', 'code'):
            tests = cls.search([(field,) + tuple(clause[1:])], limit=1)
            if tests:
                break
        if tests:
            return [(field,) + tuple(clause[1:])]
        return [(cls._rec_name,) + tuple(clause[1:])]



class Lab(ModelSQL, ModelView):
    'Lab Test'
    __name__ = 'gnuhealth.lab'

    name = fields.Char('ID', help="Lab result ID", readonly=True)
    test = fields.Many2One('gnuhealth.lab.test_type', 'Test type',
        help="Lab test type", required=True, select=True)
    patient = fields.Many2One('gnuhealth.patient', 'Patient',
     help="Patient ID", required=True, select=True)
    pathologist = fields.Many2One('gnuhealth.healthprofessional', 'Pathologist',
        help="Pathologist", select=True)
    requestor = fields.Many2One('gnuhealth.healthprofessional', 'Physician',
        help="Doctor who requested the test", select=True)
    results = fields.Text('Results')
    diagnosis = fields.Text('Diagnosis')
    critearea = fields.One2Many('gnuhealth.lab.test.critearea',
        'gnuhealth_lab_id', 'Lab Test Critearea')
    date_requested = fields.DateTime('Date requested', required=True,
        select=True)
    date_analysis = fields.DateTime('Date of the Analysis', select=True)
    request_order = fields.Integer('Request', readonly=True)

    @classmethod
    def __setup__(cls):
        super(Lab, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('id_uniq', Unique(t, t.name),
             'The test ID code must be unique')
        ]

    @staticmethod
    def default_date_requested():
        return datetime.now()

    @staticmethod
    def default_date_analysis():
        return datetime.now()

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('name'):
                config = Config(1)
                values['name'] = Sequence.get_id(
                    config.lab_sequence.id)

        return super(Lab, cls).create(vlist)


class GnuHealthLabTestUnits(ModelSQL, ModelView):
    'Lab Test Units'
    __name__ = 'gnuhealth.lab.test.units'

    name = fields.Char('Unit', select=True)
    code = fields.Char('Code', select=True)

    @classmethod
    def __setup__(cls):
        super(GnuHealthLabTestUnits, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('name_uniq', Unique(t, t.name),
             'The Unit name must be unique')
        ]


    @classmethod
    def check_xml_record(cls, records, values):
        return True


class GnuHealthTestCritearea(ModelSQL, ModelView):
    'Lab Test Critearea'
    __name__ = 'gnuhealth.lab.test.critearea'

    name = fields.Char('Analyte', required=True, select=True,
        translate=True)
    excluded = fields.Boolean('Excluded', help='Select this option when'
        ' this analyte is excluded from the test')
    result = fields.Float('Value')
    result_text = fields.Char('Result - Text', help='Non-numeric results. For '
        'example qualitative values, morphological, colors ...')
    remarks = fields.Char('Remarks')
    normal_range = fields.Text('Reference')
    lower_limit = fields.Float('Lower Limit')
    upper_limit = fields.Float('Upper Limit')
    warning = fields.Boolean('Warn', help='Warns the patient about this '
        ' analyte result'
        ' It is useful to contextualize the result to each patient status '
        ' like age, sex, comorbidities, ...')
    units = fields.Many2One('gnuhealth.lab.test.units', 'Units')
    test_type_id = fields.Many2One('gnuhealth.lab.test_type', 'Test type',
     select=True)
    gnuhealth_lab_id = fields.Many2One('gnuhealth.lab', 'Test Cases',
        select=True)
    sequence = fields.Integer('Sequence')

    # Show the warning icon if warning is active on the analyte line
    lab_warning_icon = \
        fields.Function(fields.Char('Lab Warning Icon'),
         'get_lab_warning_icon')
    
    def get_lab_warning_icon(self, name):
        if (self.warning):
            return 'gnuhealth-warning'

    @classmethod
    def __setup__(cls):
        super(GnuHealthTestCritearea, cls).__setup__()
        cls._order.insert(0, ('sequence', 'ASC'))

    @staticmethod
    def default_sequence():
        return 1

    @staticmethod
    def default_excluded():
        return False

    @fields.depends('result', 'lower_limit', 'upper_limit')
    def on_change_with_warning(self):
        if (self.result < self.lower_limit or self.result > self.upper_limit):
            return True
        return False

    @classmethod
    def check_xml_record(cls, records, values):
        return True


class GnuHealthPatientLabTest(ModelSQL, ModelView):
    'Patient Lab Test'
    __name__ = 'gnuhealth.patient.lab.test'
    
    name = fields.Many2One('gnuhealth.lab.test_type', 'Test Type',
        required=True, select=True)
    date = fields.DateTime('Date', select=True)
    state = fields.Selection([
        ('draft', 'Draft'),
        ('tested', 'Tested'),
        ('ordered', 'Ordered'),
        ('cancel', 'Cancel'),
        ], 'State', readonly=True, select=True)
    patient_id = fields.Many2One('gnuhealth.patient', 'Patient', required=True,
     select=True)
    doctor_id = fields.Many2One('gnuhealth.healthprofessional', 'Doctor',
        help="Doctor who Request the lab test.", select=True)
    request = fields.Integer('Order', readonly=True)
    urgent = fields.Boolean('Urgent')

    @classmethod
    def __setup__(cls):
        super(GnuHealthPatientLabTest, cls).__setup__()
        cls._order.insert(0, ('date', 'DESC'))
        cls._order.insert(1, ('request', 'DESC'))
        cls._order.insert(2, ('name', 'ASC'))

    @staticmethod
    def default_date():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'draft'

    @staticmethod
    def default_doctor_id():
        User = Pool().get('res.user')
        user = User(Transaction().user)
        uid = int(user.id)

        parties = Pool().get('party.party').search([
                ('internal_user', '=', uid)])
        if parties:
            doctors = Pool().get('gnuhealth.healthprofessional').search([
                    ('name', '=', parties[0].id)])
            if doctors:
                return doctors[0].id
        else:
            return False

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('request'):
                config = Config(1)
                values['request'] = Sequence.get_id(
                    config.lab_request_sequence.id)

        return super(GnuHealthPatientLabTest, cls).create(vlist)

    @classmethod
    def copy(cls, tests, default=None):
        if default is None:
            default = {}
        default = default.copy()
        default['request'] = None
        default['date'] = cls.default_date()
        return super(GnuHealthPatientLabTest, cls).copy(tests,
            default=default)



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import pytz
from trytond.model import ModelView, ModelSQL, ModelSingleton, fields
from datetime import datetime
from trytond.pool import Pool
from trytond.transaction import Transaction
from trytond.pyson import Eval, Not, Bool, PYSONEncoder, Equal, And, Or, If


__all__ = ['GnuHealthSequences', 'PatientRounding', 'RoundingProcedure',
    'PatientAmbulatoryCare', 'AmbulatoryCareProcedure']


class GnuHealthSequences(ModelSingleton, ModelSQL, ModelView):
    "Standard Sequences for GNU Health"
    __name__ = "gnuhealth.sequences"

    ambulatory_care_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Health Ambulatory Care', domain=[
            ('code', '=', 'gnuhealth.ambulatory_care')
        ]))

    patient_rounding_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Health Rounding', domain=[
            ('code', '=', 'gnuhealth.patient.rounding')
        ]))

# Class : PatientRounding
# Assess the patient and evironment periodically
# Usually done by nurses

class PatientRounding(ModelSQL, ModelView):
    'Patient Rounding'
    __name__ = 'gnuhealth.patient.rounding'

    STATES = {'readonly': Eval('state') == 'done'}

    name = fields.Many2One('gnuhealth.inpatient.registration',
        'Registration Code', required=True, states=STATES)
    code = fields.Char('Code',  states=STATES)
    health_professional = fields.Many2One('gnuhealth.healthprofessional',
        'Health Professional', readonly=True)
    evaluation_start = fields.DateTime('Start', required=True, states=STATES)
    evaluation_end = fields.DateTime('End', readonly=True)

    state = fields.Selection([
        (None, ''),
        ('draft', 'In Progress'),
        ('done', 'Done'),
        ], 'State', readonly=True)

    environmental_assessment = fields.Char('Environment', help="Environment"
        " assessment . State any disorder in the room.",states=STATES)

    weight = fields.Integer('Weight',
        help="Measured weight, in kg",states=STATES)

    # The 6 P's of rounding
    pain = fields.Boolean('Pain',
        help="Check if the patient is in pain", states=STATES)
    pain_level = fields.Integer('Pain', help="Enter the pain level, from 1 to "
            "10", states={'invisible': ~Eval('pain'),
            'readonly': Eval('state') == 'done'})

    potty = fields.Boolean('Potty', help="Check if the patient needs to "
        "urinate / defecate", states=STATES)
    position = fields.Boolean('Position', help="Check if the patient needs to "
        "be repositioned or is unconfortable", states=STATES)
    proximity = fields.Boolean('Proximity', help="Check if personal items, "
        "water, alarm, ... are not in easy reach",states=STATES)
    pump = fields.Boolean('Pumps', help="Check if there is any issues with "
        "the pumps - IVs ... ", states=STATES)
    personal_needs = fields.Boolean('Personal needs', help="Check if the "
        "patient requests anything", states=STATES)

    # Vital Signs
    systolic = fields.Integer('Systolic Pressure',states=STATES)
    diastolic = fields.Integer('Diastolic Pressure', states=STATES)
    bpm = fields.Integer('Heart Rate',
        help='Heart rate expressed in beats per minute', states=STATES)
    respiratory_rate = fields.Integer('Respiratory Rate',
        help='Respiratory rate expressed in breaths per minute', states=STATES)
    osat = fields.Integer('Oxygen Saturation',
        help='Oxygen Saturation(arterial).', states=STATES)
    temperature = fields.Float('Temperature',
        help='Temperature in celsius', states=STATES)

    # Diuresis

    diuresis = fields.Integer('Diuresis',help="volume in ml", states=STATES)
    urinary_catheter = fields.Boolean('Urinary Catheter', states=STATES)

    #Glycemia
    glycemia = fields.Integer('Glycemia', help='Blood Glucose level', states=STATES)

    depression = fields.Boolean('Depression signs', help="Check this if the "
        "patient shows signs of depression", states=STATES)
    evolution = fields.Selection([
        (None, ''),    
        ('n', 'Status Quo'),
        ('i', 'Improving'),
        ('w', 'Worsening'),
        ], 'Evolution', help="Check your judgement of current "
        "patient condition", sort=False, states=STATES)
    round_summary = fields.Text('Round Summary', states=STATES)

    signed_by = fields.Many2One(
        'gnuhealth.healthprofessional', 'Signed by', readonly=True,
        states={'invisible': Equal(Eval('state'), 'draft')},
        help="Health Professional that signed the rounding")


    warning = fields.Boolean('Warning', help="Check this box to alert the "
        "supervisor about this patient rounding. A warning icon will be shown "
        "in the rounding list", states=STATES)
    warning_icon = fields.Function(fields.Char('Warning Icon'), 'get_warn_icon')
    procedures = fields.One2Many('gnuhealth.rounding_procedure', 'name',
        'Procedures', help="List of the procedures in this rounding. Please "
        "enter the first one as the main procedure", states=STATES)

    report_start_date = fields.Function(fields.Date('Start Date'), 
        'get_report_start_date')
    report_start_time = fields.Function(fields.Time('Start Time'), 
        'get_report_start_time')
    report_end_date = fields.Function(fields.Date('End Date'), 
        'get_report_end_date')
    report_end_time = fields.Function(fields.Time('End Time'), 
        'get_report_end_time')

    @staticmethod
    def default_health_professional():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        healthprof = HealthProf.get_health_professional()
        return healthprof

    @staticmethod
    def default_evaluation_start():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'draft'

    @classmethod
    def __setup__(cls):
        super(PatientRounding, cls).__setup__()
        cls._error_messages.update({
            'health_professional_warning':
                    'No health professional associated to this user',
        })
        cls._buttons.update({
            'end_rounding': {
                'invisible': ~Eval('state').in_(['draft']),
            }})

        cls._order.insert(0, ('evaluation_start', 'DESC'))

    @classmethod
    @ModelView.button
    def end_rounding(cls, roundings):
        # End the rounding
        pool = Pool()
        HealthProfessional = pool.get('gnuhealth.healthprofessional')
                
        # Change the state of the rounding to "Done"
        signing_hp = HealthProfessional.get_health_professional()
        
        cls.write(roundings, {
            'state': 'done',
            'signed_by': signing_hp,
            'evaluation_end': datetime.now()
            })

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('code'):
                config = Config(1)
                values['code'] = Sequence.get_id(
                    config.patient_rounding_sequence.id)
        return super(PatientRounding, cls).create(vlist)


    @classmethod
    def validate(cls, roundings):
        super(PatientRounding, cls).validate(roundings)
        for rounding in roundings:
            rounding.check_health_professional()

    def check_health_professional(self):
        if not self.health_professional:
            self.raise_user_error('health_professional_warning')

    def get_report_start_date(self, name):
        Company = Pool().get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)

        dt = self.evaluation_start
        return datetime.astimezone(dt.replace(tzinfo=pytz.utc), timezone).date()

    def get_report_start_time(self, name):
        Company = Pool().get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)

        dt = self.evaluation_start
        return datetime.astimezone(dt.replace(tzinfo=pytz.utc), timezone).time()

    def get_report_end_date(self, name):
        Company = Pool().get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)

        dt = self.evaluation_end
        return datetime.astimezone(dt.replace(tzinfo=pytz.utc), timezone).date()

    def get_report_end_time(self, name):
        Company = Pool().get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)

        dt = self.evaluation_end
        return datetime.astimezone(dt.replace(tzinfo=pytz.utc), timezone).time()

    def get_warn_icon(self, name):
        if self.warning:
            return 'gnuhealth-warning'


class RoundingProcedure(ModelSQL, ModelView):
    'Rounding - Procedure'
    __name__ = 'gnuhealth.rounding_procedure'

    name = fields.Many2One('gnuhealth.patient.rounding', 'Rounding')
    procedure = fields.Many2One('gnuhealth.procedure', 'Code', required=True,
        select=True,
        help="Procedure Code, for example ICD-10-PCS Code 7-character string")
    notes = fields.Text('Notes')


class PatientAmbulatoryCare(ModelSQL, ModelView):
    'Patient Ambulatory Care'
    __name__ = 'gnuhealth.patient.ambulatory_care'

    STATES = {'readonly': Eval('state') == 'done'}

    name = fields.Char('ID', readonly=True)
    patient = fields.Many2One('gnuhealth.patient', 'Patient',
     required=True, states=STATES)

    state = fields.Selection([
        (None, ''),
        ('draft', 'In Progress'),
        ('done', 'Done'),
        ], 'State', readonly=True)

    base_condition = fields.Many2One('gnuhealth.pathology', 'Condition',
        states=STATES)
    evaluation = fields.Many2One('gnuhealth.patient.evaluation',
        'Related Evaluation', domain=[('patient', '=', Eval('patient'))],
        depends=['patient'], states=STATES)
    ordering_professional = fields.Many2One('gnuhealth.healthprofessional',
        'Requested by', states=STATES)
    health_professional = fields.Many2One('gnuhealth.healthprofessional',
        'Health Professional', readonly=True)
    procedures = fields.One2Many('gnuhealth.ambulatory_care_procedure', 'name',
        'Procedures', states=STATES,
        help="List of the procedures in this session. Please enter the first "
        "one as the main procedure")
    session_number = fields.Integer('Session #', states=STATES)
    session_start = fields.DateTime('Start', required=True, states=STATES)

    # Vital Signs
    systolic = fields.Integer('Systolic Pressure', states=STATES)
    diastolic = fields.Integer('Diastolic Pressure', states=STATES)
    bpm = fields.Integer('Heart Rate',states=STATES,
        help='Heart rate expressed in beats per minute')
    respiratory_rate = fields.Integer('Respiratory Rate',states=STATES,
        help='Respiratory rate expressed in breaths per minute')
    osat = fields.Integer('Oxygen Saturation',states=STATES,
        help='Oxygen Saturation(arterial).')
    temperature = fields.Float('Temperature',states=STATES,
        help='Temperature in celsius')

    warning = fields.Boolean('Warning', help="Check this box to alert the "
        "supervisor about this session. A warning icon will be shown in the "
        "session list",states=STATES)
    warning_icon = fields.Function(fields.Char('Warning Icon'), 'get_warn_icon')

    #Glycemia
    glycemia = fields.Integer('Glycemia', help='Blood Glucose level',
        states=STATES)

    evolution = fields.Selection([
        (None, ''),
        ('initial', 'Initial'),
        ('n', 'Status Quo'),
        ('i', 'Improving'),
        ('w', 'Worsening'),
        ], 'Evolution', help="Check your judgement of current "
        "patient condition", sort=False, states=STATES)
    session_end = fields.DateTime('End', readonly=True)
    next_session = fields.DateTime('Next Session', states=STATES)
    session_notes = fields.Text('Notes', states=STATES)

    signed_by = fields.Many2One(
        'gnuhealth.healthprofessional', 'Signed by', readonly=True,
        states={'invisible': Equal(Eval('state'), 'draft')},
        help="Health Professional that signed the session")


    @staticmethod
    def default_health_professional():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        healthprof = HealthProf.get_health_professional()
        return healthprof

    @staticmethod
    def default_session_start():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'draft'


    @classmethod
    def __setup__(cls):
        super(PatientAmbulatoryCare, cls).__setup__()
        cls._error_messages.update({
            'health_professional_warning':
                    'No health professional associated to this user',
        })
        cls._buttons.update({
            'end_session': {
                'invisible': ~Eval('state').in_(['draft']),
            }})

        cls._order.insert(0, ('session_start', 'DESC'))

    @classmethod
    @ModelView.button
    def end_session(cls, sessions):
        # End the session and discharge the patient
        pool = Pool()
        HealthProfessional = pool.get('gnuhealth.healthprofessional')
                
        # Change the state of the session to "Done"
        signing_hp = HealthProfessional.get_health_professional()
        
        cls.write(sessions, {
            'state': 'done',
            'signed_by': signing_hp,
            'session_end': datetime.now()
            })

    @classmethod
    def validate(cls, records):
        super(PatientAmbulatoryCare, cls).validate(records)
        for record in records:
            record.check_health_professional()

    def check_health_professional(self):
        if not self.health_professional:
            self.raise_user_error('health_professional_warning')

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('name'):
                config = Config(1)
                values['name'] = Sequence.get_id(
                    config.ambulatory_care_sequence.id)
        return super(PatientAmbulatoryCare, cls).create(vlist)

    @classmethod
    def copy(cls, ambulatorycares, default=None):
        if default is None:
            default = {}
        default = default.copy()
        default['name'] = None
        default['session_start'] = cls.default_session_start()
        default['session_end'] = cls.default_session_start()
        return super(PatientAmbulatoryCare, cls).copy(ambulatorycares,
            default=default)

    def get_warn_icon(self, name):
        if self.warning:
            return 'gnuhealth-warning'


class AmbulatoryCareProcedure(ModelSQL, ModelView):
    'Ambulatory Care Procedure'
    __name__ = 'gnuhealth.ambulatory_care_procedure'

    name = fields.Many2One('gnuhealth.patient.ambulatory_care', 'Session')
    procedure = fields.Many2One('gnuhealth.procedure', 'Code', required=True,
        select=True,
        help="Procedure Code, for example ICD-10-PCS Code 7-character string")
    comments = fields.Char('Comments')
